package com.rf.fileCrack.config;

import cn.hutool.core.date.DateUtil;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.rf.fileCrack.file.dao.model.FileEntity;
import com.rf.fileCrack.file.rest.FileController;
import com.rf.fileCrack.file.service.FileService;
import com.rf.fileCrack.socket.WebSocketServer;
import com.rf.fileCrack.utils.SendEmail;
import lombok.extern.slf4j.Slf4j;
import org.apache.http.HttpResponse;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.Profile;
import org.springframework.scheduling.annotation.EnableScheduling;
import org.springframework.scheduling.annotation.Scheduled;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * @Author:zsy
 * @Date:2024/12/11 16:16:30
 * @Description:文件解密相关的定时任务信息
 */
@Configuration
@EnableScheduling
@Slf4j
public class StaticScheduleTask {

    @Autowired
    private FileService fileService;

    /**
     * 根据文件状态通知算法解密或解密完成通知用户
     */
    @Scheduled(initialDelay = 60000,fixedDelay = 60000)
    public void updateFileStatus() throws IOException {
        List<FileEntity> fileEntitys = this.fileService.findByFileStatus("1");
        for (FileEntity fileEntity : fileEntitys){
            HttpResponse response = FileController.HttpClient(fileEntity.getFilePath(),fileEntity.getId());
            if (response.getStatusLine().getStatusCode() == 200){
                fileEntity.setFileStatus("2");
                fileEntity.setUpdateTime(DateUtil.now());
                this.fileService.save(fileEntity);
            }
            //WebSocketServer.sendInfo(fileEntity.getId());
        }
        fileEntitys = this.fileService.findByFileStatus("3");
        for (FileEntity fileEntity : fileEntitys){
            //根据邮箱查询用户信息
            String email = fileEntity.getEmail();
            if (email != null) {
                SendEmail sendEmail=new SendEmail();
                //破解密码
                sendEmail.setPassword(fileEntity.getPassword());
                //破解的文件名
                String[] fileNames = fileEntity.getFilePath().split("-=-");
                String fileName = fileNames[fileNames.length-1];
                sendEmail.setFileName(fileName);
                //设置要发送的邮箱
                sendEmail.setReceiveMailAccount(email);
                try {
                    sendEmail.Send();
                    fileEntity.setFileStatus("4");
                    fileEntity.setUpdateTime(DateUtil.now());
                    this.fileService.save(fileEntity);
                } catch (Exception e) {
                    e.printStackTrace();
                }
            }
        }
    }

}
