package com.rf.fileCrack.socket;

import cn.hutool.core.date.DateUtil;
import com.alibaba.fastjson.JSONObject;
import com.rf.fileCrack.file.dao.model.FileEntity;
import com.rf.fileCrack.file.rest.FileController;
import com.rf.fileCrack.file.service.FileService;
import com.rf.fileCrack.utils.SendEmail;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import javax.annotation.PostConstruct;
import javax.websocket.*;
import javax.websocket.server.PathParam;
import javax.websocket.server.ServerEndpoint;
import java.io.IOException;
import java.util.*;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.atomic.AtomicInteger;

/**
 * @author zsy
 * @description:
 * @date 2024/12/09 14:32
 */
@Slf4j
@ServerEndpoint(value = "/websocket/{userId}")
@Component
public class WebSocketServer {

    @Autowired
    private FileService fileService;

    public static WebSocketServer webSocketServer;

    @PostConstruct
    public void init(){
        webSocketServer = this;
        webSocketServer.fileService = this.fileService;
    }

    private static  HashMap<String,String>control;
    private static List<String> fileList;
    static{
        control = new HashMap<>();
    }
    /** 记录当前在线连接数 */
    private static AtomicInteger onlineCount = new AtomicInteger(0);
    /**
     * 用来存放每个客户端对应的WebSocketServer对象
     */
    private static ConcurrentHashMap<String, WebSocketServer> webSocketMap = new ConcurrentHashMap<>();
    /**
     *与某个客户端的连接会话需要通过它来给客户端发送数据
     */
    private Session session;

    /**
     * 用来接收userId
     */
    private String userId;
    /**
     * 连接建立成功调用的方法
     */
    @OnOpen
    public void onOpen(Session session , @PathParam("userId") String userId) throws IOException {
        this.session = session;
        this.userId = userId;
        //判断用户是否已经登录对应的角色
        if(fileList==null){
            fileList = new ArrayList<>();
            fileList.add(userId);
        }
        if(webSocketMap.containsKey(userId)){
            webSocketMap.remove(userId);
            webSocketMap.put(userId,this);
        }else{
            webSocketMap.put(userId,this);
            addOnlineCount();
        }
        log.info("用户连接：{},当前在线人数为：{}",userId,getOnlineCount());
        try{
            sendMessage("连接成功");
        }catch(IOException e ) {
            log.error("用户：{},网络异常",userId);
        }
        System.out.println("onOpen-----");
        for (Map.Entry entry:control.entrySet()) {
            System.out.println(entry.getKey()+"----"+entry.getValue());
        }
    }

    /**
     * 连接关闭调用的方法
     */
    @OnClose
    public void onClose(Session session) {
        if (webSocketMap.containsKey(userId)) {
            webSocketMap.remove(userId);
            subOnlineCount();
            control.remove(userId);
        }
        System.out.println("onClose---");
        if(fileList != null ){
            fileList.remove(userId);
        }
        log.info("有一连接关闭：{}，当前在线人数为：{}", userId, webSocketMap.size());
    }

    /**
     * 收到客户端消息后调用的方法
     *
     * @param message
     *            客户端发送过来的消息
     */
    @OnMessage
    public void onMessage(String message, Session session) throws IOException {
        JSONObject respJson = new JSONObject();
        System.out.println("onMessage----");
        for (Map.Entry entry:control.entrySet()) {
            System.out.println(entry.getKey()+"----"+entry.getValue());
        }
        log.info("用户消息:" + userId + ",报文:" + message);
        //sendMessage("发送成功");
        //保存解密后密码并发邮箱通知
        /*if (!StringUtils.isEmpty(message)) {
            JSONObject jsonObject = JSONObject.parseObject(message);
            String fileId = null;
            String password = null;
            if (jsonObject.containsKey("fileId") && StringUtils.isNotBlank(jsonObject.getString("fileId"))) {
                fileId = jsonObject.getString("fileId");
                if (jsonObject.containsKey("password") && StringUtils.isNotBlank(jsonObject.getString("password"))) {
                    password = jsonObject.getString("password");
                    savePassword(fileId,password);
                    respJson.put("msg","发送成功");
                }else {
                    log.error("请求的文件password错误！");
                    respJson.put("msg","发送失败，文件password错误！");
                }
            }else {
                log.error("请求的文件ID或密码为空！");
                respJson.put("msg","发送失败，文件ID或密码为空！");
            }
            sendMessage(respJson.toJSONString());
        }*/
    }

    /**
     * 发生错误时调用
     *
     * @param session
     * @param error
     */
    @OnError
    public void onError(Session session, Throwable error) {
        log.error("用户错误:" + this.userId + ",原因:" + error.getMessage());
        error.printStackTrace();
    }

    /**
     * 实现服务器主动推送
     */
    public void sendMessage(String message) throws IOException {
        this.session.getBasicRemote().sendText(message);
    }

    public static void sendInfo(String message) throws IOException {
        log.info("消息内容：" + message);
        webSocketMap.get("10002").sendMessage(message);
    }

    public static synchronized AtomicInteger getOnlineCount() {
        return onlineCount;
    }

    public static synchronized void addOnlineCount() {
        WebSocketServer.onlineCount.getAndIncrement();
    }

    public static synchronized void subOnlineCount() {
        WebSocketServer.onlineCount.getAndDecrement();
    }

    //保存解密后密码并发邮箱通知
    /*public static void savePassword(String fileId, String password) {
        FileEntity fileEntity = webSocketServer.fileService.findById(fileId);
        //根据邮箱查询用户信息
        String email = fileEntity.getEmail();
        if (email != null) {
            SendEmail sendEmail=new SendEmail();
            //破解密码
            sendEmail.setPassword(password);
            //破解的文件名
            String[] fileNames = fileEntity.getFilePath().split("-");
            String fileName = fileNames[fileNames.length-1];
            sendEmail.setFileName(fileName);
            //设置要发送的邮箱
            sendEmail.setReceiveMailAccount(email);
            try {
                sendEmail.Send();
                fileEntity.setFileStatus("4");
            } catch (Exception e) {
                e.printStackTrace();
            }
        }else {
            fileEntity.setFileStatus("3");
        }
        fileEntity.setPassword(password);
        fileEntity.setUpdateTime(DateUtil.now());
        webSocketServer.fileService.save(fileEntity);
    }*/

}
